
import rpg
import skills

import random
import types

class RandomTreasureGenerator(object):
    def __init__(self, pc=None):
        self.pc = pc
        
    def generateRandomTreasure(self, forPC=None, fromActor=None):
        if forPC == None: forPC = self.pc
        percent = random.randint(1,100)
        loot = None
        if percent < 20:
            loot = self.generateRandomWeapon(self.getLowerLevel(forPC,
                                                                fromActor))
        elif percent < 40:
            loot = self.generateRandomArmor(self.getLowerLevel(forPC, 
                                                               fromActor))
        elif percent < 60:
            loot = self.generateRandomHealth(self.getLowerLevel(forPC,
                                                                fromActor))
        return loot
            
    def generateRandomHealth(self, level):
        # Create something good enough to restore 1/4 of HP on use,
        # Assuming 10HP / level
        return HealthPotion.createPotionFor(level * 10 / 4)
    
    def generateRandomArmor(self, level):
        torso = "Torso"
        shield = "Shield"
        typesACSlot = [ ( 'Padded Armor', 1, torso),
                        ( 'Leather Armor', 2, torso),
                        ( 'Studded Leather Armor', 3, torso),
                        ( 'Chain Shirt', 4, torso ),
                        ( 'Hide Armor', 3, torso),
                        ( 'Scale Mail', 4, torso),
                        ( 'Chainmail', 5, torso),
                        ( 'Breastplate', 5, torso),
                        ( 'Splint Mail', 6, torso),
                        ( 'Banded Mail', 6, torso),
                        ( 'Half-Plate', 7, torso),
                        ( 'Full Plate', 8, torso),
                        ( 'Buckler', 1, shield),
                        ( 'Wood Shield', 1, shield),
                        ( 'Steel Shield', 2, shield),
                        ( 'Tower Shield', 4, shield) ]
        name, ac, slot = random.choice(typesACSlot)
        bonus = random.randint(level/4, level / 2)
        return rpg.Armor(name, slot, ac, bonus)
    
    def generateRandomWeapon(self, level):
        typesD = [ ('Dagger', 4), ('Mace',6), ('Club',6), ('Heavy Mace', 8),
                  ('Morningstar', 8), ('Handaxe', 6) ,('Shortsword',6),
                  ('Battleaxe', 8), ('Flail', 8), ('Longsword', 8), 
                  ('Rapier', 6), ('Scimitar', 6), ('Warhammer', 8),
                  ('Bastard Sword', 10), ('Waraxe', 10) ]
        type = random.choice(typesD)
        bonus = random.randint(level / 4, level / 2)
        return rpg.Weapon(type[0], type[1], bonus)
            
    def getLowerLevel(self, forPC, fromActor):
        if type(forPC) == types.IntType:
            level1 = forPC
        else:
            level1 = forPC.level
        if fromActor == None:
            return level1
        return min(forPC.level, fromActor.level)

# Potions
## Health

class HealthPotion(rpg.Potion):
    HEAL_ALL = 999999
    
    def __init__(self, heal, name):
        rpg.Potion.__init__(self, name)
        self.heal = heal
        
    def apply(self, pc):
        if self.amount > 0:
            pc.heal(self.heal)
            self.amount -= 1
        
    def createPotionFor(amount):
        prefixes = ["Light", "Moderate","Serious","Critical","Deep",
                    "Gaping","Atrocious", "Mortal", "Ungodly", "Nearly All"]
        name="Cure %s Wounds Potion"
        index = (amount / 30)
        if index >= len(prefixes): index = len(prefixes) -1
        if amount == HealthPotion.HEAL_ALL:
            prefix = "All"
        else:
            prefix = prefixes[index]
            amount = (index+1) * 30
            
        return HealthPotion(amount, name%prefix)
    
    createPotionFor = staticmethod(createPotionFor)
        
HEALTH_10 = HealthPotion.createPotionFor(10)

# Skills
class UseItemSkill(skills.Skill):
    def __init__(self, name, pc, item):
        skills.Skill.__init__(self, name)
        self.item = item 
        self.pc = pc
        
    def apply(self, pc):
        # TODO: Check to see if it's in inventory, remove it, etc
        self.item.apply(pc)
        
    def isEnabled(self):
        return self.enabled and self.item.amount > 0
    
    def __str__(self):
        return self.name + "(%d)" % self.item.amount