
class Skill(object):
    def __init__(self, name):
        self.name = name
        self.enabled = True
        
    def apply(self, pc):
        raise NotImplementedError
    
    def isEnabled(self):
        return self.enabled
    
    def __str__(self):
        return self.name
        
class CooldownSkill(Skill):
    def __init__(self, name,cooldownInMS):
        Skill.__init__(self, name)
        self.maxCooldown = cooldownInMS
        self.cooldown = self.maxCooldown
        
    def apply(self, pc):
        raise NotImplementedError
            
    def isEnabled(self):
        return self.cooldown >= self.maxCooldown
    
    def reset(self):
        self.cooldown = 0
    
    def update(self, delay):
        if not self.isEnabled():
            self.cooldown += delay
    
    def __str__(self):
        if self.isEnabled():
            return self.name
        else:
            secs = (self.maxCooldown - self.cooldown) / 1000
            return self.name + " (%ds)" % secs

class PowerAttack(CooldownSkill):
    def __init__(self, adjustment):
        name = "Power Attack %d" % adjustment
        CooldownSkill.__init__(self, name, 4000+2000*adjustment)
        self.adjustment = adjustment
        
    def apply(self, pc):
        if self.isEnabled() and pc.target:
            self.reset()
            pc.tilemap.log.postMessage("%s uses %s!" % (pc.name, self.name))
            pc.attack(-self.adjustment, self.adjustment)